! netcdf_file.f90 - object-oriented netCDF file interface (file)
! vi: set sw=4:
! Copyright (C) TOYODA Eizi, 2000.  All rights reserved.
!
! 2000-01-10 ˭ıѻ	
! 2000-01-13 ˭ıѻ	̾
! 2000-01-15 ˭ıѻ	Ѥ
! 2000-01-17 ˭ıѻ	⡼ɤȤη NC_FILE 
! 2000-02-22 ˭ıѻ	Ĺʸ

module netcdf_file

    use netcdf_v3
    use netcdf_error
    use netcdf_filename
    use dc_string
    implicit none

    character(len=*), private, parameter:: RCSID = &
    & "$Id:$"

    type NC_FILE
	integer::		id
	logical::		writable
	logical::		defmode
    end type

    type(NC_FILE), parameter:: &
	& NC_FILE_CLOSED = NC_FILE(0, .FALSE., .FALSE.), &
	& NC_FILE_ERROR = NC_FILE(-1, .FALSE., .FALSE.)

    !
    ! --- 롢Ĥ ---
    !

    interface Create
	module procedure NetcdfCreateFile2
    end interface

    ! usage:
    !	type(NC_FILE):: file
    !	type(NC_ERROR):: status
    !	status = Open(file, "input.nc")
    !
    interface Open
	module procedure NetcdfOpenFile2
    end interface

    interface Close
	module procedure NetcdfCloseFile
    end interface

    !
    ! --- ⡼ѹ ---
    ! 

    interface Redefine
	module procedure NetcdfRedefineFile
    end interface

    interface EndDefmode
	module procedure NetcdfFileEndDefmode
    end interface

    !
    ! --- μ ---
    !

    interface Id
    	module procedure NetcdfFileId
    end interface

    interface Filename
	module procedure NetcdfFileFilename
    end interface

    interface operator(.error.)
	module procedure NetcdfFileError
    end interface

contains

    ! 顼å
    !
    logical function NetcdfFileError(file)
    	type(NC_FILE), intent(in)::	file
    continue
	NetcdfFileError = (file%id < 0)
    end function

    !
    ! --- / ---
    !

    ! 
    ! ¸ե뤬оõ
    function NetcdfCreateFile(path) result(result)
	type(NC_FILE):: result
	character(len=*), intent(in):: path
	integer:: status
    continue
	status = nf_create(path, NF_CLOBBER, result%id)
	if (status == NF_NOERR) then
	    result%defmode = .TRUE.
	    result%writable = .TRUE.
	    call NetcdfSaveFilename(result%id, path)
	else
	    call NetcdfSaveError(status, 'NetcdfCreateNewFile', path)
	    result = NC_FILE_ERROR
	endif
    end function

    ! 顼ؿ
    function NetcdfCreateFile2(file, path) result(result)
	type(NC_ERROR):: result
	type(NC_FILE), intent(out):: file
	character(len=*), intent(in):: path
    continue
	file = NetcdfCreateFile(path)
	if (.error. file) then
	    result = NetcdfLastError()
	else
	    result = NF_NOERR
	endif
    end function

    ! 
    !
    function NetcdfOpenFile(path, writable) result(result)
	type(NC_FILE):: result
	character(len=*), intent(in):: path
	logical, optional, intent(in):: writable
	integer:: mode, status
    continue
	mode = NF_NOWRITE
	if (present(writable) .and. writable) then
	    mode = ior(mode, NF_WRITE)
	    result%writable = .TRUE.
	else
	    result%writable = .FALSE.
	endif
	status = nf_open(path, mode, result%id)
	if (status == NF_NOERR) then
	    call NetcdfSaveFilename(Id(result), path)
	    result%defmode = .FALSE.
	else
	    call NetcdfSaveError(status, 'NetcdfOpenFile', path)
	    result = NC_FILE_ERROR
	endif
    end function

    ! 顼ؿ
    function NetcdfOpenFile2(file, path, writable) result(result)
	type(NC_ERROR):: result
	type(NC_FILE), intent(out):: file
	character(len=*), intent(in):: path
	logical, optional, intent(in):: writable
    continue
	file = NetcdfOpenFile(path, writable=writable)
	if (.error. file) then
	    result = NetcdfLastError()
	else
	    result = NF_NOERR
	endif
    end function

    ! Ĥ
    !
    function NetcdfCloseFile(file) result(result)
	type(NC_ERROR):: result
	type(NC_FILE), intent(inout):: file
    continue
	result = NC_ERR(nf_close(Id(file)), 'NetcdfCloseFile', Filename(file))
	file = NC_FILE_CLOSED
    end function

    !
    ! --- ⡼ѹ ---
    !

    function NetcdfRedefineFile(file) result(result)
	type(NC_ERROR):: result
	type(NC_FILE), intent(inout):: file
    continue
	if (file%defmode) then
	    result = NF_NOERR
	    return
	endif
	result = NC_ERR(nf_redef(Id(file)), &
	    & 'NetcdfRedefineFile', Filename(file))
	if (.error. result) return
	file%defmode = .true.
    end function

    function NetcdfFileEndDefmode(file) result(result)
	type(NC_ERROR):: result
	type(NC_FILE), intent(inout):: file
    continue
	if (.not. file%defmode) then
	    result = NF_NOERR
	    return
	endif
	result = NC_ERR(nf_enddef(Id(file)), &
	    & 'NetcdfFileEndDefmode', Filename(file))
	if (.error. result) return
	file%defmode = .false.
    end function

    !
    ! --- ե˴ؤ ---
    !

    ! ID μ
    !
    integer function NetcdfFileId(file) result(result)
	type(NC_FILE), intent(in)::	file
    continue
	result = file%id
    end function

    ! ե̾μ
    !
    function NetcdfFileFilename(file) result(result)
	type(STRING)::		result
    	type(NC_FILE), intent(in)::	file
    continue
	result = NetcdfIdToFilename(Id(file))
    end function

end module
